/*
 * fstrftime - display file information for time
 *
 * synopsis:
 *	fstrftime [+format|-l] [-a|-m|-c] [-L|-P] [-p|-f] [--] file [...]
 *
 * description:
 *	fstrftime displays the time file was last accessed or
 *	modified, of when the status was changed, with strftime(3)
 *	format. If you want to display more detail information about
 *	files, consider using stat(1) appeared in NetBSD 1.6.
 *
 * options:
 *	+format	 specify strftime(3) format, default "%Y%m%d".
 *	-l	 specify long format of "%Y%m%d%H%M%S".
 *	-a|-m|-c show accessed, modified(default) or changed time.
 *	-L|-P	 use stat(2) instead of lstat(2), and vice versa.
 *	-p|-f	 show the name of file prior or following time.
 *
 * Copyright (c) 2007 Taiji Yamada <taiji@aihara.co.jp>
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
/* not completely the same as BSD strlcpy */
#define strlcpy(d, s, z) do {\
  strncpy(d, s, z);\
  if ((z) > 0)\
    d[(z)-1] = '\0';\
} while (0)

int usage(int rc)
{
  puts("usage: fstrftime [+format|-l] [-a|-m|-c] [-L|-P] [-p|-f] [--] file [...]");
  return rc;
}
int main(int argc, char *argv[])
{
  int a, t = 'm', l = !0, f = 0, rc = 0;
  struct stat sb;
  struct tm *tm;
  char ftm[256] = "%Y%m%d", lftm[] = "%Y%m%d%H%M%S", buf[256];

  for (a=1; a<argc; a++)
    if (strcmp(argv[a], "--")==0) break;
    else if (argv[a][0] == '+') strlcpy(ftm, &argv[a][1], sizeof(ftm));
    else if (strcmp(argv[a], "-l")==0) strlcpy(ftm, lftm, sizeof(ftm));
    else if (strcmp(argv[a], "-a")==0) t = 'a';
    else if (strcmp(argv[a], "-m")==0) t = 'm';
    else if (strcmp(argv[a], "-c")==0) t = 'c';
    else if (strcmp(argv[a], "-L")==0) l = 0;
    else if (strcmp(argv[a], "-P")==0) l = !0;
    else if (strcmp(argv[a], "-p")==0) f = 1;
    else if (strcmp(argv[a], "-f")==0) f = 2;
    else if (strcmp(argv[a], "--help")==0) return usage(0);
    else if (argv[a][0] == '-') return usage(1);
    else break;
  for (; a<argc; a++)
    if ((l?lstat(argv[a], &sb):stat(argv[a], &sb)) == 0 &&
	(tm=
	 (t=='a')?localtime(&sb.st_atime):
	 (t=='m')?localtime(&sb.st_mtime):
	 (t=='c')?localtime(&sb.st_ctime):
	 NULL) &&
	strftime(buf, sizeof(buf), ftm, tm)) {
      buf[sizeof(buf)-1] = '\0';
      switch (f) {
      case 0: printf("%s\n", buf); break;
      case 1: printf("%s\t%s\n", argv[a], buf); break;
      case 2: printf("%s\t%s\n", buf, argv[a]); break;
      }
      buf[0] = '\0';
    }
    else rc = 1;
  return rc;
}
